@@ -1,21 +1,8 @@
 /*
   tre-match-approx.c - TRE approximate regex matching engine
 
-  Copyright (c) 2001-2006 Ville Laurikari <vl@iki.fi>.
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+  This software is released under a BSD-style license.
+  See the file LICENSE for details and copyright.
 
 */
 
@@ -62,7 +49,7 @@ char *alloca ();
 
 #include "tre-internal.h"
 #include "tre-match-utils.h"
-#include "regex.h"
+#include "tre.h"
 #include "xmalloc.h"
 
 #define TRE_M_COST	0
@@ -213,7 +200,6 @@ tre_set_params(tre_tnfa_approx_reach_t *reach,
     reach->params.max_err = value;
 }
 
-
 reg_errcode_t
 tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 		    tre_str_type_t type, int *match_tags,
@@ -238,9 +224,6 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 
   int prev_pos;
 
-  /* Compilation flags for this regexp. */
-  int cflags = tnfa->cflags;
-
   /* Number of tags. */
   int num_tags;
   /* The reach tables. */
@@ -297,11 +280,11 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 #ifdef TRE_USE_ALLOCA
     buf = alloca(total_bytes);
 #else /* !TRE_USE_ALLOCA */
-    buf = xmalloc(total_bytes);
+    buf = xmalloc((unsigned)total_bytes);
 #endif /* !TRE_USE_ALLOCA */
     if (!buf)
       return REG_ESPACE;
-    memset(buf, 0, total_bytes);
+    memset(buf, 0, (size_t)total_bytes);
 
     /* Allocate `tmp_tags' from `buf'. */
     tmp_tags = (void *)buf;
@@ -340,7 +323,7 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
   GET_NEXT_WCHAR();
   pos = 0;
 
-  while (1)
+  while (/*CONSTCOND*/1)
     {
       DPRINT(("%03d:%2lc/%05d\n", pos, (tre_cint_t)next_c, (int)next_c));
 
@@ -352,38 +335,38 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 	  DPRINT(("  init"));
 	  for (trans = tnfa->initial; trans->state; trans++)
 	    {
-	      int id = trans->state_id;
+	      int stateid = trans->state_id;
 
 	      /* If this state is not currently in `reach_next', add it
 		 there. */
-	      if (reach_next[id].pos < pos)
+	      if (reach_next[stateid].pos < pos)
 		{
 		  if (trans->assertions && CHECK_ASSERTIONS(trans->assertions))
 		    {
 		      /* Assertions failed, don't add this state. */
-		      DPRINT((" !%d (assert)", id));
+		      DPRINT((" !%d (assert)", stateid));
 		      continue;
 		    }
-		  DPRINT((" %d", id));
-		  reach_next[id].state = trans->state;
-		  reach_next[id].pos = pos;
+		  DPRINT((" %d", stateid));
+		  reach_next[stateid].state = trans->state;
+		  reach_next[stateid].pos = pos;
 
 		  /* Compute tag values after this transition. */
 		  for (i = 0; i < num_tags; i++)
-		    reach_next[id].tags[i] = -1;
+		    reach_next[stateid].tags[i] = -1;
 
 		  if (trans->tags)
 		    for (i = 0; trans->tags[i] >= 0; i++)
 		      if (trans->tags[i] < num_tags)
-			reach_next[id].tags[trans->tags[i]] = pos;
+			reach_next[stateid].tags[trans->tags[i]] = pos;
 
 		  /* Set the parameters, depth, and costs. */
-		  reach_next[id].params = default_params;
-		  reach_next[id].depth = 0;
+		  reach_next[stateid].params = default_params;
+		  reach_next[stateid].depth = 0;
 		  for (i = 0; i < TRE_M_LAST; i++)
-		    reach_next[id].costs[0][i] = 0;
+		    reach_next[stateid].costs[0][i] = 0;
 		  if (trans->params)
-		    tre_set_params(&reach_next[id], trans->params,
+		    tre_set_params(&reach_next[stateid], trans->params,
 				   default_params);
 
 		  /* If this is the final state, mark the exact match. */
@@ -391,7 +374,7 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 		    {
 		      match_eo = pos;
 		      for (i = 0; i < num_tags; i++)
-			match_tags[i] = reach_next[id].tags[i];
+			match_tags[i] = reach_next[stateid].tags[i];
 		      for (i = 0; i < TRE_M_LAST; i++)
 			match_costs[i] = 0;
 		    }
@@ -505,7 +488,6 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 	while (deque_end != deque_start)
 	  {
 	    tre_tnfa_approx_reach_t *reach_p;
-	    int id;
 	    int depth;
 	    int cost, cost0;
 	    tre_tnfa_transition_t *trans;
@@ -687,20 +669,7 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 
 	      if (trans->assertions
 		  && (CHECK_ASSERTIONS(trans->assertions)
-		      /* Handle character class transitions. */
-		      || ((trans->assertions & ASSERT_CHAR_CLASS)
-			  && !(cflags & REG_ICASE)
-			  && !tre_isctype((tre_cint_t)prev_c, trans->u.class))
-		      || ((trans->assertions & ASSERT_CHAR_CLASS)
-			  && (cflags & REG_ICASE)
-			  && (!tre_isctype(tre_tolower((tre_cint_t)prev_c),
-					   trans->u.class)
-			      && !tre_isctype(tre_toupper((tre_cint_t)prev_c),
-					      trans->u.class)))
-		      || ((trans->assertions & ASSERT_CHAR_CLASS_NEG)
-			  && tre_neg_char_classes_match(trans->neg_classes,
-							(tre_cint_t)prev_c,
-							cflags & REG_ICASE))))
+		      || CHECK_CHAR_CLASSES(trans, tnfa, eflags)))
 		{
 		  DPRINT(("  exact,  from %d: assert failed\n", id));
 		  continue;
@@ -713,8 +682,8 @@ tre_tnfa_run_approx(const tre_tnfa_t *tnfa, const void *string, int len,
 	      cost0 = reach[id].costs[0][TRE_M_COST];
 	      err = 0;
 
-	      if (trans->code_min > prev_c ||
-		  trans->code_max < prev_c)
+	      if (trans->code_min > (tre_cint_t)prev_c
+		  || trans->code_max < (tre_cint_t)prev_c)
 		{
 		  /* Handle substitutes.  The required character was not in
 		     the string, so match it in place of whatever was supposed
