@@ -1,21 +1,8 @@
 /*
   tre-parse.c - Regexp parser
 
-  Copyright (c) 2001-2006 Ville Laurikari <vl@iki.fi>
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+  This software is released under a BSD-style license.
+  See the file LICENSE for details and copyright.
 
 */
 
@@ -64,12 +51,16 @@
 
 
 /* Some macros for expanding \w, \s, etc. */
-static const char *tre_macros[] =
-  { "t", "\t",		   "n", "\n",		  "r", "\r",
-    "f", "\f",		   "a", "\a",		  "e", "\033",
-    "w", "[[:alnum:]_]",   "W", "[^[:alnum:]_]",  "s", "[[:space:]]",
-    "S", "[^[:space:]]",   "d", "[[:digit:]]",	  "D", "[^[:digit:]]",
-   NULL };
+static const struct tre_macro_struct {
+  const char c;
+  const char *expansion;
+} tre_macros[] =
+  { {'t', "\t"},	   {'n', "\n"},		   {'r', "\r"},
+    {'f', "\f"},	   {'a', "\a"},		   {'e', "\033"},
+    {'w', "[[:alnum:]_]"}, {'W', "[^[:alnum:]_]"}, {'s', "[[:space:]]"},
+    {'S', "[^[:space:]]"}, {'d', "[[:digit:]]"},   {'D', "[^[:digit:]]"},
+    { 0, NULL }
+  };
 
 
 /* Expands a macro delimited by `regex' and `regex_end' to `buf', which
@@ -80,39 +71,20 @@ tre_expand_macro(const tre_char_t *regex, const tre_char_t *regex_end,
 		 tre_char_t *buf, size_t buf_len)
 {
   int i;
-  size_t len = regex_end - regex;
 
   buf[0] = 0;
-  for (i = 0; tre_macros[i] != NULL; i += 2)
+  if (regex >= regex_end)
+    return;
+
+  for (i = 0; tre_macros[i].expansion; i++)
     {
-      int match = 0;
-      if (strlen(tre_macros[i]) > len)
-	continue;
-#ifdef TRE_WCHAR
-      {
-	tre_char_t tmp_wcs[64];
-	unsigned int j;
-	for (j = 0; j < strlen(tre_macros[i]) && j < elementsof(tmp_wcs); j++)
-	  tmp_wcs[j] = btowc(tre_macros[i][j]);
-	tmp_wcs[j] = 0;
-	match = wcsncmp(tmp_wcs, regex, strlen(tre_macros[i]));
-      }
-#else /* !TRE_WCHAR */
-      match = strncmp(tre_macros[i], regex, strlen(tre_macros[i]));
-#endif /* !TRE_WCHAR */
-      if (match == 0)
+      if (tre_macros[i].c == *regex)
 	{
 	  unsigned int j;
-	  DPRINT(("Expanding macro '%s' => '%s'\n",
-		  tre_macros[i], tre_macros[i + 1]));
-	  for (j = 0; tre_macros[i + 1][j] != 0 && j < buf_len; j++)
-	    {
-#ifdef TRE_WCHAR
-	      buf[j] = btowc(tre_macros[i + 1][j]);
-#else /* !TRE_WCHAR */
-	      buf[j] = tre_macros[i + 1][j];
-#endif /* !TRE_WCHAR */
-	    }
+	  DPRINT(("Expanding macro '%c' => '%s'\n",
+		  tre_macros[i].c, tre_macros[i].expansion));
+	  for (j = 0; tre_macros[i].expansion[j] && j < buf_len; j++)
+	    buf[j] = tre_macros[i].expansion[j];
 	  buf[j] = 0;
 	  break;
 	}
@@ -187,8 +159,8 @@ tre_expand_ctype(tre_mem_t mem, tre_ctype_t class, tre_ast_node_t ***items,
 static int
 tre_compare_items(const void *a, const void *b)
 {
-  tre_ast_node_t *node_a = *(tre_ast_node_t **)a;
-  tre_ast_node_t *node_b = *(tre_ast_node_t **)b;
+  const tre_ast_node_t *node_a = *(tre_ast_node_t * const *)a;
+  const tre_ast_node_t *node_b = *(tre_ast_node_t * const *)b;
   tre_literal_t *l_a = node_a->obj, *l_b = node_b->obj;
   int a_min = l_a->code_min, b_min = l_b->code_min;
 
@@ -271,6 +243,8 @@ tre_ctype_t tre_ctype(const char *name)
 /* Maximum length of character class names. */
 #define MAX_CLASS_NAME
 
+#define REST(re) (int)(ctx->re_end - (re)), (re)
+
 static reg_errcode_t
 tre_parse_bracket_items(tre_parse_ctx_t *ctx, int negate,
 			tre_ctype_t neg_classes[], int *num_neg_classes,
@@ -294,8 +268,7 @@ tre_parse_bracket_items(tre_parse_ctx_t *ctx, int negate,
 	}
       else if (*re == CHAR_RBRACKET && re > ctx->re)
 	{
-	  DPRINT(("tre_parse_bracket:	done: '%.*" STRF "'\n",
-		  ctx->re_end - re, re));
+	  DPRINT(("tre_parse_bracket:	done: '%.*" STRF "'\n", REST(re)));
 	  re++;
 	  break;
 	}
@@ -307,8 +280,7 @@ tre_parse_bracket_items(tre_parse_ctx_t *ctx, int negate,
 	  if (re + 2 < ctx->re_end
 	      && *(re + 1) == CHAR_MINUS && *(re + 2) != CHAR_RBRACKET)
 	    {
-	      DPRINT(("tre_parse_bracket:  range: '%.*" STRF "'\n",
-		      ctx->re_end - re, re));
+	      DPRINT(("tre_parse_bracket:  range: '%.*" STRF "'\n", REST(re)));
 	      min = *re;
 	      max = *(re + 2);
 	      re += 3;
@@ -329,8 +301,7 @@ tre_parse_bracket_items(tre_parse_ctx_t *ctx, int negate,
 	      char tmp_str[64];
 	      const tre_char_t *endptr = re + 2;
 	      int len;
-	      DPRINT(("tre_parse_bracket:  class: '%.*" STRF "'\n",
-		      ctx->re_end - re, re));
+	      DPRINT(("tre_parse_bracket:  class: '%.*" STRF "'\n", REST(re)));
 	      while (endptr < ctx->re_end && *endptr != CHAR_COLON)
 		endptr++;
 	      if (endptr != ctx->re_end)
@@ -339,7 +310,7 @@ tre_parse_bracket_items(tre_parse_ctx_t *ctx, int negate,
 #ifdef TRE_WCHAR
 		  {
 		    tre_char_t tmp_wcs[64];
-		    wcsncpy(tmp_wcs, re + 2, len);
+		    wcsncpy(tmp_wcs, re + 2, (size_t)len);
 		    tmp_wcs[len] = L'\0';
 #if defined HAVE_WCSRTOMBS
 		    {
@@ -353,7 +324,7 @@ tre_parse_bracket_items(tre_parse_ctx_t *ctx, int negate,
 #endif /* defined HAVE_WCSTOMBS */
 		  }
 #else /* !TRE_WCHAR */
-		  strncpy(tmp_str, re + 2, len);
+		  strncpy(tmp_str, (const char*)re + 2, len);
 #endif /* !TRE_WCHAR */
 		  tmp_str[len] = '\0';
 		  DPRINT(("  class name: %s\n", tmp_str));
@@ -377,8 +348,7 @@ tre_parse_bracket_items(tre_parse_ctx_t *ctx, int negate,
 	    }
 	  else
 	    {
-	      DPRINT(("tre_parse_bracket:   char: '%.*" STRF "'\n",
-		      ctx->re_end - re, re));
+	      DPRINT(("tre_parse_bracket:   char: '%.*" STRF "'\n", REST(re)));
 	      if (*re == CHAR_MINUS && *(re + 1) != CHAR_RBRACKET
 		  && ctx->re != re)
 		/* Two ranges are not allowed to share and endpoint. */
@@ -406,7 +376,7 @@ tre_parse_bracket_items(tre_parse_ctx_t *ctx, int negate,
 	     This is broken if there are more than two "same" characters. */
 	  if (ctx->cflags & REG_ICASE && !class && status == REG_OK && !skip)
 	    {
-	      int cmin, ccurr;
+	      tre_cint_t cmin, ccurr;
 
 	      DPRINT(("adding opposite-case counterpoints\n"));
 	      while (min <= max)
@@ -462,8 +432,7 @@ tre_parse_bracket(tre_parse_ctx_t *ctx, tre_ast_node_t **result)
 
   if (*ctx->re == CHAR_CARET)
     {
-      DPRINT(("tre_parse_bracket: negate: '%.*" STRF "'\n",
-	      ctx->re_end - ctx->re, ctx->re));
+      DPRINT(("tre_parse_bracket: negate: '%.*" STRF "'\n", REST(ctx->re)));
       negate = 1;
       ctx->re++;
     }
@@ -476,7 +445,7 @@ tre_parse_bracket(tre_parse_ctx_t *ctx, tre_ast_node_t **result)
 
   /* Sort the array if we need to negate it. */
   if (negate)
-    qsort(items, i, sizeof(*items), tre_compare_items);
+    qsort(items, (unsigned)i, sizeof(*items), tre_compare_items);
 
   curr_max = curr_min = 0;
   /* Build a union of the items in the array, negated if necessary. */
@@ -645,7 +614,7 @@ tre_parse_bound(tre_parse_ctx_t *ctx, tre_ast_node_t **result)
   /* Parse number (minimum repetition count). */
   min = -1;
   if (r < ctx->re_end && *r >= L'0' && *r <= L'9') {
-    DPRINT(("tre_parse:	  min count: '%.*" STRF "'\n", ctx->re_end - r, r));
+    DPRINT(("tre_parse:	  min count: '%.*" STRF "'\n", REST(r)));
     min = tre_parse_int(&r, ctx->re_end);
   }
 
@@ -654,7 +623,7 @@ tre_parse_bound(tre_parse_ctx_t *ctx, tre_ast_node_t **result)
   if (r < ctx->re_end && *r == CHAR_COMMA)
     {
       r++;
-      DPRINT(("tre_parse:   max count: '%.*" STRF "'\n", ctx->re_end - r, r));
+      DPRINT(("tre_parse:   max count: '%.*" STRF "'\n", REST(r)));
       max = tre_parse_int(&r, ctx->re_end);
     }
 
@@ -700,7 +669,7 @@ tre_parse_bound(tre_parse_ctx_t *ctx, tre_ast_node_t **result)
 	  switch (*r)
 	    {
 	    case CHAR_PLUS:  /* Insert limit */
-	      DPRINT(("tre_parse:   ins limit: '%.*" STRF "'\n", ctx->re_end - r, r));
+	      DPRINT(("tre_parse:   ins limit: '%.*" STRF "'\n", REST(r)));
 	      r++;
 	      limit_ins = tre_parse_int(&r, ctx->re_end);
 	      if (limit_ins < 0)
@@ -708,7 +677,7 @@ tre_parse_bound(tre_parse_ctx_t *ctx, tre_ast_node_t **result)
 	      counts_set = 1;
 	      break;
 	    case CHAR_MINUS: /* Delete limit */
-	      DPRINT(("tre_parse:   del limit: '%.*" STRF "'\n", ctx->re_end - r, r));
+	      DPRINT(("tre_parse:   del limit: '%.*" STRF "'\n", REST(r)));
 	      r++;
 	      limit_del = tre_parse_int(&r, ctx->re_end);
 	      if (limit_del < 0)
@@ -716,7 +685,7 @@ tre_parse_bound(tre_parse_ctx_t *ctx, tre_ast_node_t **result)
 	      counts_set = 1;
 	      break;
 	    case CHAR_HASH:  /* Substitute limit */
-	      DPRINT(("tre_parse: subst limit: '%.*" STRF "'\n", ctx->re_end - r, r));
+	      DPRINT(("tre_parse: subst limit: '%.*" STRF "'\n", REST(r)));
 	      r++;
 	      limit_subst = tre_parse_int(&r, ctx->re_end);
 	      if (limit_subst < 0)
@@ -724,7 +693,7 @@ tre_parse_bound(tre_parse_ctx_t *ctx, tre_ast_node_t **result)
 	      counts_set = 1;
 	      break;
 	    case CHAR_TILDE: /* Maximum number of changes */
-	      DPRINT(("tre_parse: count limit: '%.*" STRF "'\n", ctx->re_end - r, r));
+	      DPRINT(("tre_parse: count limit: '%.*" STRF "'\n", REST(r)));
 	      r++;
 	      limit_err = tre_parse_int(&r, ctx->re_end);
 	      if (limit_err < 0)
@@ -758,7 +727,7 @@ tre_parse_bound(tre_parse_ctx_t *ctx, tre_ast_node_t **result)
 	      r++;
 	      break;
 	    case L'<':
-	      DPRINT(("tre_parse:    max cost: '%.*" STRF "'\n", ctx->re_end - r, r));
+	      DPRINT(("tre_parse:    max cost: '%.*" STRF "'\n", REST(r)));
 	      r++;
 	      while (*r == L' ')
 		r++;
@@ -785,21 +754,21 @@ tre_parse_bound(tre_parse_ctx_t *ctx, tre_ast_node_t **result)
 		    {
 		    case L'i':	/* Insert cost */
 		      DPRINT(("tre_parse:    ins cost: '%.*" STRF "'\n",
-			      ctx->re_end - sr, sr));
+			      REST(sr)));
 		      r++;
 		      cost_ins = cost;
 		      costs_set = 1;
 		      break;
 		    case L'd':	/* Delete cost */
 		      DPRINT(("tre_parse:    del cost: '%.*" STRF "'\n",
-			      ctx->re_end - sr, sr));
+			      REST(sr)));
 		      r++;
 		      cost_del = cost;
 		      costs_set = 1;
 		      break;
 		    case L's':	/* Substitute cost */
 		      DPRINT(("tre_parse:  subst cost: '%.*" STRF "'\n",
-			      ctx->re_end - sr, sr));
+			      REST(sr)));
 		      r++;
 		      cost_subst = cost;
 		      costs_set = 1;
@@ -843,10 +812,18 @@ tre_parse_bound(tre_parse_ctx_t *ctx, tre_ast_node_t **result)
 
 
   /* Parse trailing '?' marking minimal repetition. */
-  if (r < ctx->re_end && *r == CHAR_QUESTIONMARK)
+  if (r < ctx->re_end)
     {
-      minimal = !(ctx->cflags & REG_UNGREEDY);
-      r++;
+      if (*r == CHAR_QUESTIONMARK)
+	{
+	  minimal = !(ctx->cflags & REG_UNGREEDY);
+	  r++;
+	}
+      else if (*r == CHAR_STAR || *r == CHAR_PLUS)
+	{
+	  /* These are reserved for future extensions. */
+	  return REG_BADRPT;
+	}
     }
 
   /* Create the AST node(s). */
@@ -870,7 +847,7 @@ tre_parse_bound(tre_parse_ctx_t *ctx, tre_ast_node_t **result)
 	 iteration node. */
       if (approx || costs_set || counts_set)
 	{
-	  unsigned int *params;
+	  int *params;
 	  tre_iteration_t *iter = (*result)->obj;
 
 	  if (costs_set || counts_set)
@@ -964,12 +941,11 @@ tre_parse(tre_parse_ctx_t *ctx)
 
   if (!ctx->nofirstsub)
     {
-      STACK_PUSH(stack, ctx->re);
-      STACK_PUSH(stack, ctx->submatch_id);
-      STACK_PUSH(stack, PARSE_MARK_FOR_SUBMATCH);
+      STACK_PUSH(stack, int, ctx->submatch_id);
+      STACK_PUSH(stack, int, PARSE_MARK_FOR_SUBMATCH);
       ctx->submatch_id++;
     }
-  STACK_PUSH(stack, PARSE_RE);
+  STACK_PUSH(stack, int, PARSE_RE);
   ctx->re_start = ctx->re;
   ctx->re_end = ctx->re + ctx->len;
 
@@ -982,7 +958,7 @@ tre_parse(tre_parse_ctx_t *ctx)
     {
       if (status != REG_OK)
 	break;
-      symbol = (tre_parse_re_stack_symbol_t)tre_stack_pop(stack);
+      symbol = tre_stack_pop_int(stack);
       switch (symbol)
 	{
 	case PARSE_RE:
@@ -992,15 +968,15 @@ tre_parse(tre_parse_ctx_t *ctx)
 	  if (!(ctx->cflags & REG_LITERAL)
 	      && ctx->cflags & REG_EXTENDED)
 #endif /* REG_LITERAL */
-	    STACK_PUSHX(stack, PARSE_UNION);
-	  STACK_PUSHX(stack, PARSE_BRANCH);
+	    STACK_PUSHX(stack, int, PARSE_UNION);
+	  STACK_PUSHX(stack, int, PARSE_BRANCH);
 	  break;
 
 	case PARSE_BRANCH:
 	  /* Parse a branch.  A branch is one or more pieces, concatenated.
 	     A piece is an atom possibly followed by a postfix operator. */
-	  STACK_PUSHX(stack, PARSE_CATENATION);
-	  STACK_PUSHX(stack, PARSE_PIECE);
+	  STACK_PUSHX(stack, int, PARSE_CATENATION);
+	  STACK_PUSHX(stack, int, PARSE_PIECE);
 	  break;
 
 	case PARSE_PIECE:
@@ -1009,8 +985,8 @@ tre_parse(tre_parse_ctx_t *ctx)
 #ifdef REG_LITERAL
 	  if (!(ctx->cflags & REG_LITERAL))
 #endif /* REG_LITERAL */
-	    STACK_PUSHX(stack, PARSE_POSTFIX);
-	  STACK_PUSHX(stack, PARSE_ATOM);
+	    STACK_PUSHX(stack, int, PARSE_POSTFIX);
+	  STACK_PUSHX(stack, int, PARSE_ATOM);
 	  break;
 
 	case PARSE_CATENATION:
@@ -1035,7 +1011,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 		    if (!(ctx->cflags & REG_EXTENDED) && depth == 0)
 		      status = REG_EPAREN;
 		    DPRINT(("tre_parse:	  group end: '%.*" STRF "'\n",
-			    ctx->re_end - ctx->re, ctx->re));
+			    REST(ctx->re)));
 		    depth--;
 		    if (!(ctx->cflags & REG_EXTENDED))
 		      ctx->re += 2;
@@ -1049,26 +1025,26 @@ tre_parse(tre_parse_ctx_t *ctx)
 	    if (ctx->cflags & REG_RIGHT_ASSOC)
 	      {
 		/* Right associative concatenation. */
-		STACK_PUSHX(stack, result);
-		STACK_PUSHX(stack, PARSE_POST_CATENATION);
-		STACK_PUSHX(stack, PARSE_CATENATION);
-		STACK_PUSHX(stack, PARSE_PIECE);
+		STACK_PUSHX(stack, voidptr, result);
+		STACK_PUSHX(stack, int, PARSE_POST_CATENATION);
+		STACK_PUSHX(stack, int, PARSE_CATENATION);
+		STACK_PUSHX(stack, int, PARSE_PIECE);
 	      }
 	    else
 #endif /* REG_RIGHT_ASSOC */
 	      {
 		/* Default case, left associative concatenation. */
-		STACK_PUSHX(stack, PARSE_CATENATION);
-		STACK_PUSHX(stack, result);
-		STACK_PUSHX(stack, PARSE_POST_CATENATION);
-		STACK_PUSHX(stack, PARSE_PIECE);
+		STACK_PUSHX(stack, int, PARSE_CATENATION);
+		STACK_PUSHX(stack, voidptr, result);
+		STACK_PUSHX(stack, int, PARSE_POST_CATENATION);
+		STACK_PUSHX(stack, int, PARSE_PIECE);
 	      }
 	    break;
 	  }
 
 	case PARSE_POST_CATENATION:
 	  {
-	    tre_ast_node_t *tree = tre_stack_pop(stack);
+	    tre_ast_node_t *tree = tre_stack_pop_voidptr(stack);
 	    tre_ast_node_t *tmp_node;
 	    tmp_node = tre_ast_new_catenation(ctx->mem, tree, result);
 	    if (!tmp_node)
@@ -1088,11 +1064,11 @@ tre_parse(tre_parse_ctx_t *ctx)
 	    {
 	    case CHAR_PIPE:
 	      DPRINT(("tre_parse:	union: '%.*" STRF "'\n",
-		      ctx->re_end - ctx->re, ctx->re));
-	      STACK_PUSHX(stack, PARSE_UNION);
-	      STACK_PUSHX(stack, result);
-	      STACK_PUSHX(stack, PARSE_POST_UNION);
-	      STACK_PUSHX(stack, PARSE_BRANCH);
+		      REST(ctx->re)));
+	      STACK_PUSHX(stack, int, PARSE_UNION);
+	      STACK_PUSHX(stack, voidptr, result);
+	      STACK_PUSHX(stack, int, PARSE_POST_UNION);
+	      STACK_PUSHX(stack, int, PARSE_BRANCH);
 	      ctx->re++;
 	      break;
 
@@ -1108,7 +1084,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 	case PARSE_POST_UNION:
 	  {
 	    tre_ast_node_t *tmp_node;
-	    tre_ast_node_t *tree = tre_stack_pop(stack);
+	    tre_ast_node_t *tree = tre_stack_pop_voidptr(stack);
 	    tmp_node = tre_ast_new_union(ctx->mem, tree, result);
 	    if (!tmp_node)
 	      return REG_ESPACE;
@@ -1126,41 +1102,55 @@ tre_parse(tre_parse_ctx_t *ctx)
 #endif /* REG_LITERAL */
 	  switch (*ctx->re)
 	    {
-	    case CHAR_STAR:
 	    case CHAR_PLUS:
 	    case CHAR_QUESTIONMARK:
+	      if (!(ctx->cflags & REG_EXTENDED))
+		break;
+		/*FALLTHROUGH*/
+	    case CHAR_STAR:
 	      {
 		tre_ast_node_t *tmp_node;
 		int minimal = (ctx->cflags & REG_UNGREEDY) ? 1 : 0;
 		int rep_min = 0;
 		int rep_max = -1;
+#ifdef TRE_DEBUG
 		const tre_char_t *tmp_re;
+#endif
 
 		if (*ctx->re == CHAR_PLUS)
 		  rep_min = 1;
 		if (*ctx->re == CHAR_QUESTIONMARK)
 		  rep_max = 1;
+#ifdef TRE_DEBUG
 		tmp_re = ctx->re;
-		
-		if (ctx->re + 1 < ctx->re_end
-		    && *(ctx->re + 1) == CHAR_QUESTIONMARK)
-		  { 
-		    minimal = !(ctx->cflags & REG_UNGREEDY);
-		    ctx->re++;
+#endif
+
+		if (ctx->re + 1 < ctx->re_end)
+		  {
+		    if (*(ctx->re + 1) == CHAR_QUESTIONMARK)
+		      {
+			minimal = !(ctx->cflags & REG_UNGREEDY);
+			ctx->re++;
+		      }
+		    else if (*(ctx->re + 1) == CHAR_STAR
+			     || *(ctx->re + 1) == CHAR_PLUS)
+		      {
+			/* These are reserved for future extensions. */
+			return REG_BADRPT;
+		      }
 		  }
 
 		DPRINT(("tre_parse: %s star: '%.*" STRF "'\n",
-			minimal ? "  minimal" : "greedy",
-			ctx->re_end - tmp_re, tmp_re));
+			minimal ? "  minimal" : "greedy", REST(tmp_re)));
 		ctx->re++;
 		tmp_node = tre_ast_new_iter(ctx->mem, result, rep_min, rep_max,
 					    minimal);
 		if (tmp_node == NULL)
 		  return REG_ESPACE;
 		result = tmp_node;
-		STACK_PUSHX(stack, PARSE_POSTFIX);
-		break;
+		STACK_PUSHX(stack, int, PARSE_POSTFIX);
 	      }
+	      break;
 
 	    case CHAR_BACKSLASH:
 	      /* "\{" is special without REG_EXTENDED */
@@ -1181,13 +1171,13 @@ tre_parse(tre_parse_ctx_t *ctx)
 
 	    parse_brace:
 	      DPRINT(("tre_parse:	bound: '%.*" STRF "'\n",
-		      ctx->re_end - ctx->re, ctx->re));
+		      REST(ctx->re)));
 	      ctx->re++;
 
 	      status = tre_parse_bound(ctx, &result);
 	      if (status != REG_OK)
 		return status;
-	      STACK_PUSHX(stack, PARSE_POSTFIX);
+	      STACK_PUSHX(stack, int, PARSE_POSTFIX);
 	      break;
 	    }
 	  break;
@@ -1218,14 +1208,14 @@ tre_parse(tre_parse_ctx_t *ctx)
 		  int new_cflags = ctx->cflags;
 		  int bit = 1;
 		  DPRINT(("tre_parse:	extension: '%.*" STRF "\n",
-			  ctx->re_end - ctx->re, ctx->re));
+			  REST(ctx->re)));
 		  ctx->re += 2;
-		  while (1)
+		  while (/*CONSTCOND*/1)
 		    {
 		      if (*ctx->re == L'i')
 			{
 			  DPRINT(("tre_parse:	    icase: '%.*" STRF "\n",
-				  ctx->re_end - ctx->re, ctx->re));
+				  REST(ctx->re)));
 			  if (bit)
 			    new_cflags |= REG_ICASE;
 			  else
@@ -1235,7 +1225,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 		      else if (*ctx->re == L'n')
 			{
 			  DPRINT(("tre_parse:	  newline: '%.*" STRF "\n",
-				  ctx->re_end - ctx->re, ctx->re));
+				  REST(ctx->re)));
 			  if (bit)
 			    new_cflags |= REG_NEWLINE;
 			  else
@@ -1246,7 +1236,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 		      else if (*ctx->re == L'r')
 			{
 			  DPRINT(("tre_parse: right assoc: '%.*" STRF "\n",
-				  ctx->re_end - ctx->re, ctx->re));
+				  REST(ctx->re)));
 			  if (bit)
 			    new_cflags |= REG_RIGHT_ASSOC;
 			  else
@@ -1258,7 +1248,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 		      else if (*ctx->re == L'U')
 			{
 			  DPRINT(("tre_parse:    ungreedy: '%.*" STRF "\n",
-				  ctx->re_end - ctx->re, ctx->re));
+				  REST(ctx->re)));
 			  if (bit)
 			    new_cflags |= REG_UNGREEDY;
 			  else
@@ -1269,14 +1259,14 @@ tre_parse(tre_parse_ctx_t *ctx)
 		      else if (*ctx->re == CHAR_MINUS)
 			{
 			  DPRINT(("tre_parse:	 turn off: '%.*" STRF "\n",
-				  ctx->re_end - ctx->re, ctx->re));
+				  REST(ctx->re)));
 			  ctx->re++;
 			  bit = 0;
 			}
 		      else if (*ctx->re == CHAR_COLON)
 			{
 			  DPRINT(("tre_parse:	 no group: '%.*" STRF "\n",
-				  ctx->re_end - ctx->re, ctx->re));
+				  REST(ctx->re)));
 			  ctx->re++;
 			  depth++;
 			  break;
@@ -1284,8 +1274,8 @@ tre_parse(tre_parse_ctx_t *ctx)
 		      else if (*ctx->re == CHAR_HASH)
 			{
 			  DPRINT(("tre_parse:    comment: '%.*" STRF "\n",
-				  ctx->re_end - ctx->re, ctx->re));
-			  /* A comment can contain any character except a 
+				  REST(ctx->re)));
+			  /* A comment can contain any character except a
 			     right parenthesis */
 			  while (*ctx->re != CHAR_RPAREN
 				 && ctx->re < ctx->re_end)
@@ -1309,9 +1299,9 @@ tre_parse(tre_parse_ctx_t *ctx)
 
 		  /* Turn on the cflags changes for the rest of the
 		     enclosing group. */
-		  STACK_PUSHX(stack, ctx->cflags);
-		  STACK_PUSHX(stack, PARSE_RESTORE_CFLAGS);
-		  STACK_PUSHX(stack, PARSE_RE);
+		  STACK_PUSHX(stack, int, ctx->cflags);
+		  STACK_PUSHX(stack, int, PARSE_RESTORE_CFLAGS);
+		  STACK_PUSHX(stack, int, PARSE_RE);
 		  ctx->cflags = new_cflags;
 		  break;
 		}
@@ -1326,24 +1316,22 @@ tre_parse(tre_parse_ctx_t *ctx)
 		      && *(ctx->re + 2) == CHAR_COLON)
 		    {
 		      DPRINT(("tre_parse: group begin: '%.*" STRF
-			      "', no submatch\n",
-			      ctx->re_end - ctx->re, ctx->re));
+			      "', no submatch\n", REST(ctx->re)));
 		      /* Don't mark for submatching. */
 		      ctx->re += 3;
-		      STACK_PUSHX(stack, PARSE_RE);
+		      STACK_PUSHX(stack, int, PARSE_RE);
 		    }
 		  else
 		    {
 		      DPRINT(("tre_parse: group begin: '%.*" STRF
-			      "', submatch %d\n",
-			      ctx->re_end - ctx->re, ctx->re,
+			      "', submatch %d\n", REST(ctx->re),
 			      ctx->submatch_id));
 		      ctx->re++;
 		      /* First parse a whole RE, then mark the resulting tree
 			 for submatching. */
-		      STACK_PUSHX(stack, ctx->submatch_id);
-		      STACK_PUSHX(stack, PARSE_MARK_FOR_SUBMATCH);
-		      STACK_PUSHX(stack, PARSE_RE);
+		      STACK_PUSHX(stack, int, ctx->submatch_id);
+		      STACK_PUSHX(stack, int, PARSE_MARK_FOR_SUBMATCH);
+		      STACK_PUSHX(stack, int, PARSE_RE);
 		      ctx->submatch_id++;
 		    }
 		}
@@ -1357,7 +1345,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 		      && *(ctx->re - 1) == CHAR_BACKSLASH))
 		{
 		  DPRINT(("tre_parse:	    empty: '%.*" STRF "'\n",
-			  ctx->re_end - ctx->re, ctx->re));
+			  REST(ctx->re)));
 		  /* We were expecting an atom, but instead the current
 		     subexpression was closed.	POSIX leaves the meaning of
 		     this to be implementation-defined.	 We interpret this as
@@ -1374,7 +1362,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 
 	    case CHAR_LBRACKET: /* bracket expression */
 	      DPRINT(("tre_parse:     bracket: '%.*" STRF "'\n",
-		      ctx->re_end - ctx->re, ctx->re));
+		      REST(ctx->re)));
 	      ctx->re++;
 	      status = tre_parse_bracket(ctx, &result);
 	      if (status != REG_OK)
@@ -1390,7 +1378,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 		      || *(ctx->re + 1) == CHAR_RPAREN))
 		{
 		  ctx->re++;
-		  STACK_PUSHX(stack, PARSE_ATOM);
+		  STACK_PUSHX(stack, int, PARSE_ATOM);
 		  break;
 		}
 
@@ -1424,17 +1412,16 @@ tre_parse(tre_parse_ctx_t *ctx)
 	      if (*(ctx->re + 1) == L'Q')
 		{
 		  DPRINT(("tre_parse: tmp literal: '%.*" STRF "'\n",
-			  ctx->re_end - ctx->re, ctx->re));
+			  REST(ctx->re)));
 		  ctx->cflags |= REG_LITERAL;
 		  temporary_cflags |= REG_LITERAL;
 		  ctx->re += 2;
-		  STACK_PUSHX(stack, PARSE_ATOM);
+		  STACK_PUSHX(stack, int, PARSE_ATOM);
 		  break;
 		}
 #endif /* REG_LITERAL */
 
-	      DPRINT(("tre_parse:  bleep: '%.*" STRF "'\n",
-		      ctx->re_end - ctx->re, ctx->re));
+	      DPRINT(("tre_parse:  bleep: '%.*" STRF "'\n", REST(ctx->re)));
 	      ctx->re++;
 	      switch (*ctx->re)
 		{
@@ -1466,7 +1453,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 		      char tmp[3] = {0, 0, 0};
 		      long val;
 		      DPRINT(("tre_parse:  8 bit hex: '%.*" STRF "'\n",
-			      ctx->re_end - ctx->re + 2, ctx->re - 2));
+			      REST(ctx->re - 2)));
 
 		      if (tre_isxdigit(ctx->re[0]) && ctx->re < ctx->re_end)
 			{
@@ -1479,8 +1466,8 @@ tre_parse(tre_parse_ctx_t *ctx)
 			  ctx->re++;
 			}
 		      val = strtol(tmp, NULL, 16);
-		      result = tre_ast_new_literal(ctx->mem, val, val,
-						   ctx->position);
+		      result = tre_ast_new_literal(ctx->mem, (int)val,
+						   (int)val, ctx->position);
 		      ctx->position++;
 		      break;
 		    }
@@ -1507,11 +1494,12 @@ tre_parse(tre_parse_ctx_t *ctx)
 		      ctx->re++;
 		      tmp[i] = 0;
 		      val = strtol(tmp, NULL, 16);
-		      result = tre_ast_new_literal(ctx->mem, val, val,
+		      result = tre_ast_new_literal(ctx->mem, (int)val, (int)val,
 						   ctx->position);
 		      ctx->position++;
 		      break;
 		    }
+		  /*FALLTHROUGH*/
 
 		default:
 		  if (tre_isdigit(*ctx->re))
@@ -1519,7 +1507,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 		      /* Back reference. */
 		      int val = *ctx->re - L'0';
 		      DPRINT(("tre_parse:     backref: '%.*" STRF "'\n",
-			      ctx->re_end - ctx->re + 1, ctx->re - 1));
+			      REST(ctx->re - 1)));
 		      result = tre_ast_new_literal(ctx->mem, BACKREF, val,
 						   ctx->position);
 		      if (result == NULL)
@@ -1532,7 +1520,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 		    {
 		      /* Escaped character. */
 		      DPRINT(("tre_parse:     escaped: '%.*" STRF "'\n",
-			      ctx->re_end - ctx->re + 1, ctx->re - 1));
+			      REST(ctx->re - 1)));
 		      result = tre_ast_new_literal(ctx->mem, *ctx->re, *ctx->re,
 						   ctx->position);
 		      ctx->position++;
@@ -1546,7 +1534,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 
 	    case CHAR_PERIOD:	 /* the any-symbol */
 	      DPRINT(("tre_parse:	  any: '%.*" STRF "'\n",
-		      ctx->re_end - ctx->re, ctx->re));
+		      REST(ctx->re)));
 	      if (ctx->cflags & REG_NEWLINE)
 		{
 		  tre_ast_node_t *tmp1;
@@ -1585,7 +1573,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 		  || ctx->re == ctx->re_start)
 		{
 		  DPRINT(("tre_parse:	      BOL: '%.*" STRF "'\n",
-			  ctx->re_end - ctx->re, ctx->re));
+			  REST(ctx->re)));
 		  result = tre_ast_new_literal(ctx->mem, ASSERTION,
 					       ASSERT_AT_BOL, -1);
 		  if (result == NULL)
@@ -1606,7 +1594,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 		  || ctx->re + 1 == ctx->re_end)
 		{
 		  DPRINT(("tre_parse:	      EOL: '%.*" STRF "'\n",
-			  ctx->re_end - ctx->re, ctx->re));
+			  REST(ctx->re)));
 		  result = tre_ast_new_literal(ctx->mem, ASSERTION,
 					       ASSERT_AT_EOL, -1);
 		  if (result == NULL)
@@ -1624,11 +1612,11 @@ tre_parse(tre_parse_ctx_t *ctx)
 		  && *ctx->re == CHAR_BACKSLASH && *(ctx->re + 1) == L'E')
 		{
 		  DPRINT(("tre_parse:	 end tmps: '%.*" STRF "'\n",
-			  ctx->re_end - ctx->re, ctx->re));
+			  REST(ctx->re)));
 		  ctx->cflags &= ~temporary_cflags;
 		  temporary_cflags = 0;
 		  ctx->re += 2;
-		  STACK_PUSHX(stack, PARSE_PIECE);
+		  STACK_PUSHX(stack, int, PARSE_PIECE);
 		  break;
 		}
 
@@ -1654,7 +1642,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 		       && *(ctx->re + 1) == CHAR_LBRACE)))
 		{
 		  DPRINT(("tre_parse:	    empty: '%.*" STRF "'\n",
-			  ctx->re_end - ctx->re, ctx->re));
+			  REST(ctx->re)));
 		  result = tre_ast_new_literal(ctx->mem, EMPTY, -1, -1);
 		  if (!result)
 		    return REG_ESPACE;
@@ -1662,7 +1650,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 		}
 
 	      DPRINT(("tre_parse:     literal: '%.*" STRF "'\n",
-		      ctx->re_end - ctx->re, ctx->re));
+		      REST(ctx->re)));
 	      /* Note that we can't use an tre_isalpha() test here, since there
 		 may be characters which are alphabetic but neither upper or
 		 lower case. */
@@ -1709,7 +1697,7 @@ tre_parse(tre_parse_ctx_t *ctx)
 
 	case PARSE_MARK_FOR_SUBMATCH:
 	  {
-	    int submatch_id = (int)tre_stack_pop(stack);
+	    int submatch_id = tre_stack_pop_int(stack);
 
 	    if (result->submatch_id >= 0)
 	      {
@@ -1729,7 +1717,11 @@ tre_parse(tre_parse_ctx_t *ctx)
 	  }
 
 	case PARSE_RESTORE_CFLAGS:
-	  ctx->cflags = (int)tre_stack_pop(stack);
+	  ctx->cflags = tre_stack_pop_int(stack);
+	  break;
+
+	default:
+	  assert(0);
 	  break;
 	}
     }
