@@ -1,21 +1,8 @@
 /*
   agrep.c - Approximate grep
 
-  Copyright (c) 2001-2006 Ville Laurikari <vl@iki.fi>.
-
-  This software is free; you can redistribute it and/or modify it
-  under the terms of the GNU Lesser General Public License as
-  published by the Free Software Foundation; either version 2.1 of the
-  License, or (at your option) any later version.
-
-  This software is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this software; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+  This software is released under a BSD-style license.
+  See the file LICENSE for details and copyright.
 
 */
 
@@ -33,9 +20,19 @@
 #include <assert.h>
 #include <limits.h>
 #include <unistd.h>
-#include "getopt.h"
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif /* HAVE_GETOPT_H */
 #include "regex.h"
-#include "gettext.h"
+
+#ifdef HAVE_GETTEXT
+#include <libintl.h>
+#else
+#define gettext(s) s
+#define bindtextdomain(p, d)
+#define textdomain(p)
+#endif
+
 #define _(String) gettext(String)
 
 #undef MAX
@@ -45,11 +42,12 @@
 
 /* Short options. */
 static char const short_options[] =
-"cd:e:hiklnqsvwyBD:E:HI:MS:V0123456789";
+"cd:e:hiklnqsvwyBD:E:HI:MS:V0123456789-:";
 
 static int show_help;
 char *program_name;
 
+#ifdef HAVE_GETOPT_LONG
 /* Long options that have no corresponding short equivalents. */
 enum {
   COLOR_OPTION = CHAR_MAX + 1,
@@ -88,6 +86,7 @@ static struct option const long_options[] =
   {"word-regexp", no_argument, NULL, 'w'},
   {0, 0, 0, 0}
 };
+#endif /* HAVE_GETOPT_LONG */
 
 static void
 tre_agrep_usage(int status)
@@ -97,7 +96,7 @@ tre_agrep_usage(int status)
       fprintf(stderr, _("Usage: %s [OPTION]... PATTERN [FILE]...\n"),
 	      program_name);
       fprintf(stderr, _("Try `%s --help' for more information.\n"),
-	      program_name);
+              program_name);
     }
   else
     {
@@ -196,7 +195,7 @@ static regaparams_t match_params;
 
 /* The color string used with the --color option.  If set, the
    environment variable GREP_COLOR overrides this default value. */
-static char *highlight = "01;31";
+static const char *highlight = "01;31";
 
 /* Sets `record' to the next complete record from file `fd', and `record_len'
    to the length of the record.	 Returns 1 when there are no more records,
@@ -263,7 +262,7 @@ tre_agrep_get_next_record(int fd, const char *filename)
 	}
 
       /* Find the next record delimiter. */
-      errcode = regnexec(&delim, next_record, data_len - (next_record - buf),
+      errcode = tre_regnexec(&delim, next_record, data_len - (next_record - buf),
 			 1, pmatch, 0);
 
 
@@ -378,7 +377,7 @@ tre_agrep_handle_file(const char *filename)
 	break;
 
       /* See if the record matches. */
-      errcode = reganexec(&preg, record, record_len, &match, match_params, 0);
+      errcode = tre_reganexec(&preg, record, record_len, &match, match_params, 0);
       if ((!invert_match && errcode == REG_OK)
 	  || (invert_match && errcode == REG_NOMATCH))
 	{
@@ -473,7 +472,7 @@ main(int argc, char **argv)
   int comp_flags = REG_EXTENDED;
   char *tmp_str;
   char *regexp = NULL;
-  char *delim_regexp = "\n";
+  const char *delim_regexp = "\n";
   int word_regexp = 0;
   int literal_string = 0;
   int max_cost_set = 0;
@@ -495,13 +494,17 @@ main(int argc, char **argv)
   print_filename = -1;
   print_cost = 0;
   be_silent = 0;
-  regaparams_default(&match_params);
+  tre_regaparams_default(&match_params);
   match_params.max_cost = 0;
 
   /* Parse command line options. */
   while (1)
     {
+#ifdef HAVE_GETOPT_LONG
       c = getopt_long(argc, argv, short_options, long_options, NULL);
+#else /* !HAVE_GETOPT_LONG */
+      c = getopt(argc, argv, short_options);
+#endif /* !HAVE_GETOPT_LONG */
       if (c == -1)
 	break;
 
@@ -594,11 +597,8 @@ main(int argc, char **argv)
 	    char *version;
 	    tre_config(TRE_CONFIG_VERSION, &version);
 	    printf("%s (TRE agrep) %s\n\n", program_name, version);
-	    printf(_("Copyright (c) 2002-2006 Ville Laurikari.\n"));
 	    printf(_("\
-This is free software; see the source for copying conditions. There is NO\n\
-warranty; not even for MERCHANTABILITY or FITNESS FOR A \
-PARTICULAR PURPOSE.\n"));
+Copyright (c) 2001-2009 Ville Laurikari <vl@iki.fi>.\n"));
 	    printf("\n");
 	    exit(0);
 	    break;
@@ -606,12 +606,33 @@ PARTICULAR PURPOSE.\n"));
 	case '?':
 	  /* Ambiguous match or extraneous parameter. */
 	  break;
+
+	case '-':
+	  /* Emulate some long options on systems which don't
+	     have getopt_long. */
+	  if (strcmp(optarg, "color") == 0
+	      || strcmp(optarg, "colour") == 0)
+	    color_option = 1;
+	  else if (strcmp(optarg, "show-position") == 0)
+	    print_position = 1;
+	  else if (strcmp(optarg, "help") == 0)
+	    show_help = 1;
+	  else
+	    {
+	      fprintf(stderr, _("%s: invalid option --%s\n"),
+		      program_name, optarg);
+	      exit(2);
+	    }
+	  break;
+
+#ifdef HAVE_GETOPT_LONG
 	case COLOR_OPTION:
 	  color_option = 1;
 	  break;
 	case SHOW_POSITION_OPTION:
 	  print_position = 1;
 	  break;
+#endif /* HAVE_GETOPT_LONG */
 	case 0:
 	  /* Long options without corresponding short options. */
 	  break;
@@ -719,28 +740,28 @@ PARTICULAR PURPOSE.\n"));
     }
 
   /* Compile the pattern. */
-  errcode = regcomp(&preg, regexp, comp_flags);
+  errcode = tre_regcomp(&preg, regexp, comp_flags);
   if (errcode)
     {
       char errbuf[256];
-      regerror(errcode, &preg, errbuf, sizeof(errbuf));
+      tre_regerror(errcode, &preg, errbuf, sizeof(errbuf));
       fprintf(stderr, "%s: %s: %s\n",
 	      program_name, _("Error in search pattern"), errbuf);
       return 2;
     }
 
   /* Compile the record delimiter pattern. */
-  errcode = regcomp(&delim, delim_regexp, REG_EXTENDED | REG_NEWLINE);
+  errcode = tre_regcomp(&delim, delim_regexp, REG_EXTENDED | REG_NEWLINE);
   if (errcode)
     {
       char errbuf[256];
-      regerror(errcode, &preg, errbuf, sizeof(errbuf));
+      tre_regerror(errcode, &preg, errbuf, sizeof(errbuf));
       fprintf(stderr, "%s: %s: %s\n",
 	      program_name, _("Error in record delimiter pattern"), errbuf);
       return 2;
     }
 
-  if (regexec(&delim, "", 0, NULL, 0) == REG_OK)
+  if (tre_regexec(&delim, "", 0, NULL, 0) == REG_OK)
     {
       fprintf(stderr, "%s: %s\n", program_name,
 	      _("Record delimiter pattern must not match an empty string"));
