@@ -1,20 +1,8 @@
 /*
   randtest.c - tests with random regexps
 
-  Copyright (C) 2001, 2002 Ville Laurikari <vl@iki.fi>
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License version 2 (June
-  1991) as published by the Free Software Foundation.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  This software is released under a BSD-style license.
+  See the file LICENSE for details and copyright.
 
 */
 
@@ -77,9 +65,9 @@ main(int argc, char **argv)
 	{
 	  buf = malloc(sizeof(*buf) * len);
 	  strncpy(buf, regex, len - 1);
-	  status = regncomp(&preg, buf, len, flags);
+	  status = tre_regncomp(&preg, buf, len, flags);
 	  if (status == REG_OK)
-	    regfree(&preg);
+	    tre_regfree(&preg);
 	}
       printf("\n");
     }
