@@ -1,21 +1,8 @@
 /*
   retest.c - TRE regression test program
 
-  Copyright (c) 2001-2006 Ville Laurikari <vl@iki.fi>.
-
-  This software is free; you can redistribute it and/or modify it
-  under the terms of the GNU Lesser General Public License as
-  published by the Free Software Foundation; either version 2.1 of the
-  License, or (at your option) any later version.
-
-  This software is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this software; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+  This software is released under a BSD-style license.
+  See the file LICENSE for details and copyright.
 
 */
 
@@ -49,14 +36,77 @@
 #define xmalloc malloc
 #define xfree free
 #endif /* !TRE_VERSION */
+#include "tre-internal.h"
 
-#define elementsof(x)	(sizeof(x)/sizeof(x[0]))
+#ifdef WRETEST
+#include <wchar.h>
+#define CHAR_T wchar_t
+#define L(x) (L ## x)
+
+#define MAXSTRSIZE 1024
+static wchar_t wstr[MAXSTRSIZE];
+static wchar_t wregex[MAXSTRSIZE];
+static int woffs[MAXSTRSIZE];
+
+#define tre_regexec tre_regwexec
+#define tre_regnexec tre_regwnexec
+#define tre_regcomp tre_regwcomp
+#define tre_regncomp tre_regwncomp
+
+/* Iterate mbrtowc over the multi-byte sequence STR of length LEN,
+   store the result in BUF and memoize the successive byte offsets
+   in OFF.  */
+
+static int
+mbntowc (wchar_t *buf, const char *str, size_t len, int *off)
+{
+  int n, wlen;
+#ifdef HAVE_MBSTATE_T
+  mbstate_t cst;
+  memset(&cst, 0, sizeof(cst));
+#endif
+
+  if (len >= MAXSTRSIZE)
+    {
+      fprintf(stderr, "Increase MAXSTRSIZE to %ld or more and recompile!\n",
+	      (long)len + 1);
+      exit(EXIT_FAILURE);
+    }
+
+  if (off)
+    {
+      memset(off + 1, -1, len * sizeof(int));
+      *off = 0;
+    }
+
+  wlen = 0;
+  while (len > 0)
+    {
+      n = tre_mbrtowc(buf ? buf++ : NULL, str, len, &cst);
+      if (n < 0)
+	return n;
+      if (n == 0)
+	n = 1;
+      str += n;
+      len -= n;
+      wlen += 1;
+      if (off)
+	*(off += n) = wlen;
+    }
+
+  return(wlen);
+}
+
+#else /* !WRETEST */
+#define CHAR_T char
+#define L(x) (x)
+#endif /* !WRETEST */
 
 static int valid_reobj = 0;
 static regex_t reobj;
-static regmatch_t pmatch[32];
-static char *regex_pattern;
-static int cflags;
+static regmatch_t pmatch_global[32];
+static const CHAR_T *regex_pattern;
+static int cflags_global;
 static int use_regnexec = 0;
 static int use_regncomp = 0;
 static int avoid_eflags = 0;
@@ -84,26 +134,26 @@ test_status(char c)
 
 
 static int
-wrap_regexec(const char *data, size_t len,
+wrap_regexec(const CHAR_T *data, size_t len,
 	     size_t pmatch_len, regmatch_t *pmatch, int eflags)
 {
-  char *buf = NULL;
+  CHAR_T *buf = NULL;
   int result;
 
   if (len == 0 && use_regnexec)
     {
-      /* Zero length string and using regnexec(), the pointer we give
+      /* Zero length string and using tre_regnexec(), the pointer we give
 	 should not be dereferenced at all. */
       buf = NULL;
     }
   else
     {
       /* Copy the data to a separate buffer to make a better test for
-	 regexec() and regnexec(). */
-      buf = xmalloc(len + !use_regnexec);
+	 tre_regexec() and tre_regnexec(). */
+      buf = xmalloc((len + !use_regnexec) * sizeof(CHAR_T));
       if (!buf)
 	return REG_ESPACE;
-      memcpy(buf, data, len);
+      memcpy(buf, data, len * sizeof(CHAR_T));
       test_status('#');
     }
 
@@ -111,15 +161,15 @@ wrap_regexec(const char *data, size_t len,
   if (use_regnexec)
     {
       if (len == 0)
-	result = regnexec(&reobj, NULL, len, pmatch_len, pmatch, eflags);
+	result = tre_regnexec(&reobj, NULL, len, pmatch_len, pmatch, eflags);
       else
-	result = regnexec(&reobj, buf, len, pmatch_len, pmatch, eflags);
+	result = tre_regnexec(&reobj, buf, len, pmatch_len, pmatch, eflags);
     }
   else
 #endif /* HAVE_REGNEXEC */
     {
-      buf[len] = '\0';
-      result = regexec(&reobj, buf, pmatch_len, pmatch, eflags);
+      buf[len] = L('\0');
+      result = tre_regexec(&reobj, buf, pmatch_len, pmatch, eflags);
     }
 
   xfree(buf);
@@ -127,21 +177,22 @@ wrap_regexec(const char *data, size_t len,
 }
 
 static int
-wrap_regcomp(regex_t *preg, const char *data, size_t len, int cflags)
+wrap_regcomp(regex_t *preg, const CHAR_T *data, size_t len, int cflags)
 {
 #ifdef HAVE_REGNCOMP
   if (use_regncomp)
-    return regncomp(preg, data, len, cflags);
+    return tre_regncomp(preg, data, len, cflags);
   else
-    return regcomp(preg, data, cflags);
+    return tre_regcomp(preg, data, cflags);
 #else /* !HAVE_REGNCOMP */
   fprintf(stderr, "%s\n", data);
-  return regcomp(preg, data, cflags);
+  return tre_regcomp(preg, data, cflags);
 #endif /* !HAVE_REGNCOMP */
 }
 
 static int
-execute(char *data, int len, size_t pmatch_len, regmatch_t *pmatch, int eflags)
+execute(const CHAR_T *data, int len, size_t pmatch_len, regmatch_t *pmatch,
+	int eflags)
 {
 #ifdef MALLOC_DEBUGGING
   int i = 0;
@@ -168,17 +219,23 @@ execute(char *data, int len, size_t pmatch_len, regmatch_t *pmatch, int eflags)
 }
 
 static int
-check(va_list ap, int ret, char *str, size_t pmatch_len, regmatch_t *pmatch,
-      int eflags)
+check(va_list ap, int ret, const CHAR_T *str,
+      size_t pmatch_len, regmatch_t *pmatch, int eflags)
 {
   int fail = 0;
 
   if (ret != va_arg(ap, int))
     {
+#ifndef WRETEST
       printf("Exec error, regex: \"%s\", cflags %d, "
-	     "string: \"%s\", eflags %d\n", regex_pattern, cflags,
+	     "string: \"%s\", eflags %d\n", regex_pattern, cflags_global,
 	     str, eflags);
-      printf("	got %smatch (regexec returned %d)\n", ret ? "no " : "", ret);
+#else /* WRETEST */
+      printf("Exec error, regex: \"%ls\", cflags %d, "
+	     "string: \"%ls\", eflags %d\n", regex_pattern, cflags_global,
+	     str, eflags);
+#endif /* WRETEST */
+      printf("	got %smatch (tre_regexec returned %d)\n", ret ? "no " : "", ret);
       return 1;
     }
 
@@ -193,13 +250,34 @@ check(va_list ap, int ret, char *str, size_t pmatch_len, regmatch_t *pmatch,
 	  if (rm_so == END)
 	    break;
 	  rm_eo = va_arg(ap, int);
+#ifdef WRETEST
+	  if (rm_so >= 0)
+	    {
+	      int n = rm_so;
+
+	      if ((rm_so = woffs[rm_so]) < 0 ||
+		  (n = rm_eo, rm_eo = woffs[rm_eo]) < 0)
+		{
+		  printf("Invalid or incomplete multi-byte sequence "
+			 "in string %ls before byte offset %d\n", str, n);
+		  return 1;
+		}
+	    }
+#endif /* WRETEST */
 	  if (pmatch[i].rm_so != rm_so
 	      || pmatch[i].rm_eo != rm_eo)
 	    {
+#ifndef WRETEST
 	      printf("Exec error, regex: \"%s\", string: \"%s\"\n",
 		     regex_pattern, str);
 	      printf("	group %d: expected (%d, %d) \"%.*s\", "
 		     "got (%d, %d) \"%.*s\"\n",
+#else /* WRETEST */
+	      printf("Exec error, regex: \"%ls\", string: \"%ls\"\n",
+		     regex_pattern, str);
+	      printf("	group %d: expected (%d, %d) \"%.*ls\", "
+		     "got (%d, %d) \"%.*ls\"\n",
+#endif /* WRETEST */
 		     i, rm_so, rm_eo, rm_eo - rm_so, str + rm_so,
 		     (int)pmatch[i].rm_so, (int)pmatch[i].rm_eo,
 		     (int)(pmatch[i].rm_eo - pmatch[i].rm_so),
@@ -208,10 +286,14 @@ check(va_list ap, int ret, char *str, size_t pmatch_len, regmatch_t *pmatch,
 	    }
 	}
 
-      if (!(cflags & REG_NOSUB) && reobj.re_nsub != i - 1
+      if (!(cflags_global & REG_NOSUB) && reobj.re_nsub != i - 1
 	  && reobj.re_nsub <= pmatch_len && pmatch)
 	{
+#ifndef WRETEST
 	  printf("Comp error, regex: \"%s\"\n", regex_pattern);
+#else /* WRETEST */
+	  printf("Comp error, regex: \"%ls\"\n", regex_pattern);
+#endif /* WRETEST */
 	  printf("  re_nsub is %d, should be %d\n", (int)reobj.re_nsub, i - 1);
 	  fail = 1;
 	}
@@ -221,8 +303,13 @@ check(va_list ap, int ret, char *str, size_t pmatch_len, regmatch_t *pmatch,
 	if (pmatch[i].rm_so != -1 || pmatch[i].rm_eo != -1)
 	  {
 	    if (!fail)
+#ifndef WRETEST
 	      printf("Exec error, regex: \"%s\", string: \"%s\"\n",
 		     regex_pattern, str);
+#else /* WRETEST */
+	      printf("Exec error, regex: \"%ls\", string: \"%ls\"\n",
+		     regex_pattern, str);
+#endif /* WRETEST */
 	    printf("  group %d: expected (-1, -1), got (%d, %d)\n",
 		   i, (int)pmatch[i].rm_so, (int)pmatch[i].rm_eo);
 	    fail = 1;
@@ -234,12 +321,12 @@ check(va_list ap, int ret, char *str, size_t pmatch_len, regmatch_t *pmatch,
 
 
 static void
-test_nexec(char *data, size_t len, int eflags, ...)
+test_nexec(const char *data, size_t len, int eflags, ...)
 {
   int m;
   int fail = 0;
   int extra_flags[] = {0, REG_BACKTRACKING_MATCHER, REG_APPROX_MATCHER};
-  int i;
+  size_t i;
   va_list ap;
 
   if (!valid_reobj)
@@ -248,6 +335,21 @@ test_nexec(char *data, size_t len, int eflags, ...)
       return;
     }
 
+#ifdef WRETEST
+      {
+	int wlen = mbntowc(wstr, data, len, woffs);
+	if (wlen < 0)
+	  {
+	    exec_errors++;
+	    printf("Invalid or incomplete multi-byte sequence in %s\n", data);
+	    return;
+	  }
+	wstr[wlen] = L'\0';
+	len = wlen;
+      }
+#define data wstr
+#endif /* WRETEST */
+
   use_regnexec = 1;
 
   for (i = 0; i < elementsof(extra_flags); i++)
@@ -263,9 +365,11 @@ test_nexec(char *data, size_t len, int eflags, ...)
 
       /* Test with a pmatch array. */
       exec_tests++;
-      m = execute(data, len, elementsof(pmatch), pmatch, final_flags);
+      m = execute(data, len, elementsof(pmatch_global), pmatch_global,
+		  final_flags);
       va_start(ap, eflags);
-      fail |= check(ap, m, data, elementsof(pmatch), pmatch, final_flags);
+      fail |= check(ap, m, data, elementsof(pmatch_global), pmatch_global,
+		    final_flags);
       va_end(ap);
 
       /* Same test with a NULL pmatch. */
@@ -276,6 +380,10 @@ test_nexec(char *data, size_t len, int eflags, ...)
       va_end(ap);
     }
 
+#ifdef WRETEST
+#undef data
+#endif /* WRETEST */
+
   if (fail)
     exec_errors++;
 }
@@ -283,7 +391,7 @@ test_nexec(char *data, size_t len, int eflags, ...)
 
 
 static void
-test_exec(char *str, int eflags, ...)
+test_exec(const char *str, int eflags, ...)
 {
   int m;
   int fail = 0;
@@ -292,7 +400,7 @@ test_exec(char *str, int eflags, ...)
 		       REG_BACKTRACKING_MATCHER,
 		       REG_APPROX_MATCHER,
 		       REG_BACKTRACKING_MATCHER | REG_APPROX_MATCHER};
-  int i;
+  size_t i;
   va_list ap;
 
   if (!valid_reobj)
@@ -301,6 +409,21 @@ test_exec(char *str, int eflags, ...)
       return;
     }
 
+#ifdef WRETEST
+      {
+	int wlen = mbntowc(wstr, str, len, woffs);
+	if (wlen < 0)
+	  {
+	    exec_errors++;
+	    printf("Invalid or incomplete multi-byte sequence in %s\n", str);
+	    return;
+	  }
+	wstr[wlen] = L'\0';
+	len = wlen;
+      }
+#define str wstr
+#endif /* WRETEST */
+
   for (use_regnexec = 0; use_regnexec < 2; use_regnexec++)
     {
       for (i = 0; i < elementsof(extra_flags); i++)
@@ -316,9 +439,11 @@ test_exec(char *str, int eflags, ...)
 
 	  /* Test with a pmatch array. */
 	  exec_tests++;
-	  m = execute(str, len, elementsof(pmatch), pmatch, final_flags);
+	  m = execute(str, len, elementsof(pmatch_global), pmatch_global,
+		      final_flags);
 	  va_start(ap, eflags);
-	  fail |= check(ap, m, str, elementsof(pmatch), pmatch, final_flags);
+	  fail |= check(ap, m, str, elementsof(pmatch_global), pmatch_global,
+			final_flags);
 	  va_end(ap);
 
 	  /* Same test with a NULL pmatch. */
@@ -330,27 +455,47 @@ test_exec(char *str, int eflags, ...)
 	}
     }
 
+#ifdef WRETEST
+#undef str
+#endif /* WRETEST */
+
   if (fail)
     exec_errors++;
 }
 
 
 static void
-test_comp(char *re, int flags, int ret)
+test_comp(const char *re, int flags, int ret)
 {
   int errcode = 0;
   int len = strlen(re);
-  regex_pattern = re;
-  cflags = flags;
-
-  comp_tests++;
 
   if (valid_reobj)
     {
-      regfree(&reobj);
+      tre_regfree(&reobj);
       valid_reobj = 0;
     }
 
+  comp_tests++;
+
+#ifdef WRETEST
+  {
+    int wlen = mbntowc(wregex, re, len, NULL);
+
+    if (wlen < 0)
+      {
+	comp_errors++;
+	printf("Invalid or incomplete multi-byte sequence in %s\n", re);
+	return;
+      }
+    wregex[wlen] = L'\0';
+    len = wlen;
+  }
+#define re wregex
+#endif /* WRETEST */
+  regex_pattern = re;
+  cflags_global = flags;
+
 #ifdef MALLOC_DEBUGGING
   {
     static int j = 0;
@@ -377,9 +522,17 @@ test_comp(char *re, int flags, int ret)
   errcode = wrap_regcomp(&reobj, re, len, flags);
 #endif /* !MALLOC_DEBUGGING */
 
+#ifdef WRETEST
+#undef re
+#endif /* WRETEST */
+
   if (errcode != ret)
     {
+#ifndef WRETEST
       printf("Comp error, regex: \"%s\"\n", regex_pattern);
+#else /* WRETEST */
+      printf("Comp error, regex: \"%ls\"\n", regex_pattern);
+#endif /* WRETEST */
       printf("	expected return code %d, got %d.\n",
 	     ret, errcode);
       comp_errors++;
@@ -390,10 +543,21 @@ test_comp(char *re, int flags, int ret)
 }
 
 
+
+/* To enable tests for known bugs, set this to 1. */
+#define KNOWN_BUG 0
+
 int
 main(int argc, char **argv)
 {
 
+#ifdef WRETEST
+  /* Need an 8-bit locale.  Or move the two tests with non-ascii
+     characters to the localized internationalization tests.  */
+  if (setlocale(LC_CTYPE, "en_US.ISO-8859-1") == NULL)
+    fprintf(stderr, "Could not set locale en_US.ISO-8859-1.  Expect some\n"
+		    "`Invalid or incomplete multi-byte sequence' errors.\n");
+#endif /* WRETEST */
   /* Large number of macros in one regexp. */
   test_comp("[A-Z]\\d\\s?\\d[A-Z]{2}|[A-Z]\\d{2}\\s?\\d[A-Z]{2}|[A-Z]{2}\\d"
 	    "\\s?\\d[A-Z]{2}|[A-Z]{2}\\d{2}\\s?\\d[A-Z]{2}|[A-Z]\\d[A-Z]\\s?"
@@ -407,7 +571,7 @@ main(int argc, char **argv)
   test_comp("^!pfast [0-9]{1,15} ([0-9]{1,3}\\.){3}[0-9]{1,3}[0-9]{1,5}$",
 	    REG_EXTENDED, 0);
 
-#if 0
+#if KNOWN_BUG
   /* Should these match or not? */
   test_comp("(a)*-\\1b", REG_EXTENDED, 0);
   test_exec("aaa-b", 0, REG_NOMATCH);
@@ -546,11 +710,12 @@ main(int argc, char **argv)
   test_comp("(a*)+", REG_EXTENDED, 0);
   test_exec("-", 0, REG_OK, 0, 0, 0, 0, END);
 
-#if 0
+  /* This test blows up the backtracking matcher. */
+  avoid_eflags = REG_BACKTRACKING_MATCHER;
   test_comp("((a*)*b)*b", REG_EXTENDED, 0);
   test_exec("aaaaaaaaaaaaaaaaaaaaaaaaab", 0, REG_OK,
 	    25, 26, -1, -1, -1, -1, END);
-#endif
+  avoid_eflags = 0;
 
   test_comp("", 0, 0);
   test_exec("", 0, REG_OK, 0, 0, END);
@@ -618,7 +783,7 @@ main(int argc, char **argv)
   test_exec("foozot", 0, REG_OK, 0, 6, 3, 3, -1, -1, END);
   test_exec("foobarzot", 0, REG_OK, 0, 9, 3, 6, 3, 6, END);
   test_exec("foobarbarzot", 0, REG_OK, 0, 12, 3, 9, 6, 9, END);
-#if 0
+
   test_comp("foo((zup)*|(bar)*|(zap)*)*zot", REG_EXTENDED, 0);
   test_exec("foobarzapzot", 0, REG_OK,
 	    0, 12, 6, 9, -1, -1, -1, -1, 6, 9, END);
@@ -632,7 +797,6 @@ main(int argc, char **argv)
 	    0, 9, 3, 6, -1, -1, -1, -1, 3, 6, END);
   test_exec("foozot", 0, REG_OK,
 	    0, 6, 3, 3, -1, -1, -1, -1, -1, -1, END);
-#endif
 
 
   /* Test case where, e.g., Perl and Python regexp functions, and many
@@ -1121,6 +1285,8 @@ main(int argc, char **argv)
   test_exec("aax xxa", 0, REG_OK, 2, 3, END);
   test_comp("\\Bx\\b", REG_EXTENDED, 0);
   test_exec("aax xxx", 0, REG_OK, 2, 3, END);
+  test_comp("\\<.", REG_EXTENDED, 0);
+  test_exec(";xaa", 0, REG_OK, 1, 2, END);
 
   /* Shorthands for character classes. */
   test_comp("\\w+", REG_EXTENDED, 0);
@@ -1357,8 +1523,7 @@ main(int argc, char **argv)
 
   test_comp("\\(a*\\)*\\(x\\)\\(\\1\\)", 0, 0);
   test_exec("x", 0, REG_OK, 0, 1, 0, 0, 0, 1, 1, 1, END);
-#if 0
-  /* This test fails currently. */
+#if KNOWN_BUG
   test_exec("ax", 0, REG_OK, 0, 2, 1, 1, 1, 2, 2, 2, END);
 #endif
 
@@ -1368,9 +1533,9 @@ main(int argc, char **argv)
   test_comp("((.*)\\1)+", REG_EXTENDED, 0);
   test_exec("aa", 0, REG_OK, 0, 2, 0, 2, 0, 1, END);
 
-#if 0
+#if KNOWN_BUG
   test_comp("()(\\1\\1)*", REG_EXTENDED, 0);
-  test_exec("", 0, REG_OK, 0, 0, 0, 0, -1, -1, END);
+  test_exec("", 0, REG_OK, 0, 0, 0, 0, 0, 0, END);
 #endif
 
   /* Check that back references work with REG_NOSUB. */
@@ -1469,9 +1634,9 @@ main(int argc, char **argv)
   test_exec("ξޤϡ", 0, REG_OK, 10, 13, END);
 
 #if !defined(WIN32) && !defined(__OpenBSD__)
-  if (setlocale(LC_CTYPE, "fi_FI.ISO-8859-1") != NULL)
+  if (setlocale(LC_CTYPE, "en_US.ISO-8859-1") != NULL)
     {
-      printf("\nTesting LC_CTYPE fi_FI.ISO-8859-1\n");
+      printf("\nTesting LC_CTYPE en_US.ISO-8859-1\n");
       test_comp("aBCdeFghiJKlmnoPQRstuvWXyZ", REG_ICASE, 0);
       test_exec("abCDefGhiJKlmNoPqRStuVwXyz", 0, REG_OK, 0, 29, END);
     }
@@ -1494,7 +1659,7 @@ main(int argc, char **argv)
 #endif /* TRE_MULTIBYTE */
 #endif
 
-  regfree(&reobj);
+  tre_regfree(&reobj);
 
   printf("\n");
   if (comp_errors || exec_errors)
